## Example of some analyses in 'pavo' (V 0.5-1)
## Created 25/11/2013
## Updated 9/12/2014
## Thomas White

# This is to highlight some (not all!) of the analyses of spectral data that can
# be run using 'pavo'. Note that there are probably tidier ways of doing just 
# about everything in here.
# There are two data sets used here. 'Butterfly' is a set of reflectance spectra
# from 30 female and 30 male Eurema hecabe (UV + yellow) butterflies, while 
# 'lomandra' contains 50 reflectance spectra from Lomandra longifolia (a straight-up green plant) 

# Clear workspace
rm(list=ls()) 

# Packages
library(pavo)
library(ggplot2)

# setwd('')  # Set working directory if need be

##--------------------------------------------------------------------------##
                      ## Import data

# For pavo, the first column of your spectral data file should be named 'wl' and
# contain the wavelengths. The other colums are your spectra. Pavo can also handle 
# some of those awful propietary file-types like '.ttt' (curse your oceanoptics!) 
# as well as text files, with the function 'getspec'. 

# Eurema hecabe (butterflies) reflectance specs
  butterfly <- read.csv('butterfly.csv')

# Green plants (lomandra) reflectance specs
  plant <- read.csv('lomandra.csv')

##--------------------------------------------------------------------------##
                     ## Binning, Trimming, Tidying

# Convert to 'rspec' objects, interpolate in 1 nm intervals (default), trim range
# 300 - 700 nm. This may not be necessary if you use 'getspec' to...get your specs
  plant <- as.rspec(plant, lim = c(300, 700))
  butterfly <- as.rspec(butterfly, lim = c(300, 700))

# Calculate mean plant spectra
  plant.mean <- aggspec(plant)   # plants 
  names(plant.mean) <- c('wl', 'plant')  # Rename the columns

# Aggregate butterfly spectra by sex 
# 'By = 30' works here because there are 30 spectra per sex, but you can also 
# use regular expressions for uneven sampling
  butterfly.mean <- aggspec(butterfly, by = 30)  
  names(butterfly.mean) <- c('wl', 'female', 'male')

# Visually inspect the effect of different levels of LOESS smoothing. When smoothing,
# you just want to take the edge off the noise without changing the shape of 
# the curve. The default 3 x 3 plotting window is a bug, and I'm pretty sure it
# has been fixed in the dev version. 
  plotsmooth(butterfly.mean, minsmooth = 0.05, maxsmooth = 0.5, curves = 5, ask = FALSE)

# Apply selected smoothing function. Can also use procspec for a bunch of 
# other things (normalising, fixing negative values, centering etc.) in the same
# function call. Just don't use 'fixneg' and 'smooth' in the same function call
# as there's currently a bug. If you want to use both of them, first use 'fixneg',
# then 'smooth', in separate functions.
  butterfly <- procspec(butterfly, opt = 'smooth', span = 0.15)
  plant <- procspec(plant, opt = 'smooth', span = 0.15)

# Another eg. using 'procspec' to look at a normalised data set, which can be 
# nice for visualising the shape of spectra independant of magnitude differences
  data.norm <- procspec(cbind(butterfly.mean, plant.mean[,-1]), opt = c('minimum', 'maximum'))
  plot(data.norm)

# Chop butterfly data into sexes (or whatever functional groups you're interested in).
# I just find it a bit easier to handle this way.
  butt.male <- as.rspec(cbind(butterfly$wl, butterfly[grep('m', colnames(butterfly), fixed = TRUE, value = TRUE)]))
  butt.female <- as.rspec(cbind(butterfly$wl, butterfly[grep('f', colnames(butterfly), fixed = TRUE, value = TRUE)]))

##--------------------------------------------------------------------------##
                    ## Visualising/summarising spectra

# Plot all raw spectra in an overlay plot (default)
  plot(butterfly)

# Individually plot several spectra for easy comparison. Rapidly becomes silly 
# if too many spectra are specified, hence only 3 being used here. 
# Alternately, save it as a very long image if you want more. 
  plot(butterfly[,1:4], type = 'stack')

# A heat map. This is only really useful when data are '3d'. Eg. when you have 
# reflectance spectra of an iridescent sample at various angles. These butterfly
# data are only 2d so it's not super-informative here.
  plot(butterfly, type = 'heatmap')

# Actually, here's a quick example of the heatmap option using data built into
# pavo. This eg is in the package vignette.
  data(teal)  # Iridescent reflectance spectra from a green-winged teal
  angles <- seq(15, 70, by = 5)  # Angles at which spectra were recorded
  teal.sm <- procspec(teal, opt = 'smooth')  # Smooth it  
  plot(teal.sm, type = 'heatmap', varying = angles,
       ylab = expression(paste("Incident angle (", degree, ")")),
       las = 1, useRaster = TRUE)

# Back to the butterflies!
# Plot aggregated spectra - this is nice. By default it plots the mean +- s.d, 
# but you can specify what you like.
  aggplot(butt.female, lcol = 'black', shadecol = 'lightblue', lwd = 1.3, ylim = c(0, 100), alpha = 0.45)  # Females
  par(new = TRUE)
  aggplot(butt.male, lcol = 'black', shadecol = 'red', lwd = 1.3, ylim = c(0, 100), alpha = 0.45)  # Males
  par(new = TRUE)
  aggplot(plant, lcol = 'black', shadecol = 'forestgreen', lwd = 1.3, ylim = c(0, 100), alpha = 0.45)  # Plants

##--------------------------------------------------------------------------##
                                ## Analysing                

# Simple 'tristimulus' (hue, saturation, brightness) summary of spectra - 
# see last page of 'pavo' vignette for what the 23 (!) variables actually mean (important!), 
# and be sure to know what you want.
  female.summary <- summary(butt.female)
  male.summary <- summary(butt.male)
  plant.summary <- summary(plant)

# Simple example analysis - Is there sexual dichromatism in UV 'brightness'?
# Define restricted wavelength range for UV calculations
  rowLow <- which(butt.female$wl == 300)
  rowHigh <- which(butt.female$wl == 420)

# Summarise tristimulus variables for UV range only. It'll throw some errors 
# beacuse of the restricted wavelength range here.
  female.UV <- summary(as.rspec(butt.female[rowLow:rowHigh,]))
  male.UV <- summary(as.rspec(butt.female[rowLow:rowHigh,]))

# Test for sex differences in UV brightness. Of course we would normally check
# our statistical assumptions etc first...
  m1 <- aov(female.UV$B1 ~ male.UV$B1)
  summary(m1)  # Yep, males are brighter in the UV  

##--------------------------------------------------------------------------##
                 ## Visual modelling - segment analysis

# Segment classification of Endler 1990. It's not widely used anymore, but 
# can be useful in some circumstances.
  fem.seg <- segclass(butt.female)
  male.seg <- segclass(butt.male)
  plant.seg <- segclass(plant)
  plot(fem.seg, col = 'red', pch = 20, xlim = c(0, 1), ylim = c(0, 1))
  points(male.seg, col = 'blue', pch = 20)
  points(plant.seg, col = 'forestgreen', pch = 20)

##--------------------------------------------------------------------------##   
                                ## Visual modelling 

## Vorobyev et al.'s (1998) 'receptor noise'

# This was designed for 'discriminatory' questions with chromatically similar stimuli
# being viewed by known receivers (e.g. is my animal visible against a 
# typical background to predator x?). So it isn't really an appropriate question/analysis
# given the current data, as yellow butterflies against green backgrounds will
# obviously be readily discriminable. As with all colour-analyses, it's important
# to match the question you're asking with the most appropriate method. E.g. 
# for 'discriminatory' questions, consider the receptor-noise model, for 
# 'perceptual' questions, consider a colour-space approach like the tetrahedron 
# (as below).

# That said, lets use the receptor-noise model on these data for the hell of it.

# Save the average background spectrum separately without the wavelength column. 
# Can make things easier.
  back <- plant.mean[,-1]  

# Tack the 'background' spectrum on to the end of both male and female data sets.
# This just makes it easier to specify contrasts with only the background later on.
# Receptor-noise uses a pairwise approach (e.g. butterfly 1 vs background, 
# butterfly 2 vs background) when calculating contrasts, hence the use of a single
# average background here rather than all 50 plant spectra. Biologically, averaging
# background spectra can rapidly become meaningless if the background is composed of
# lots of differently coloured stuff (e.g. leaf litter). Here we can get away with it
# because the background (green leaves) is super-homogeneous. Otherwise you'd need 
# to look at taking measures of background irradience, for example.
  female.RN <- cbind(butt.female, back)
  male.RN <- cbind(butt.male, back)

## BLUE TIT visual system ##

# pavo currently has a number of bird visual systems built in. For other viewers, 
# you can use 'sensmodel' to model custom visual systems (see below). Note that all 
# of these options are important and can have substantial effects on the results,
# make sure you know what they mean! Here we're log-transforming our quantum catch
# (qcatch = 'fi'),in line with the Weber-Fechner law, and we're not using 
# von Kries correction (default) because it's unnecessary here. We're using standard
# daylight ('D65') as an illuminant.
# Also, for receptor-noise models, make sure relative = FALSE. i.e. you DO NOT want 
# relative quantum catch values calculated (that's for colour-spaces, as below). 
  female.bt <- vismodel(female.RN, visual = 'bt', achromatic = 'bt.dc', 
                        illum = 'D65', relative = FALSE, qcatch = 'fi')
  male.bt <- vismodel(male.RN, visual = 'bt', achromatic = 'bt.dc', 
                      illum = 'D65', relative = FALSE, qcatch = 'fi')

# Calculate chromatic and achromatic contrasts (colour-distances). Units are 
# 'just noticable differences' (JND's). Subset = 'back' as we're only interested
# in contrast against the background, not contrasts against other butterflies. 
  female.btdist <- coldist(female.bt, subset = 'back', vis = 'tetra', achro = TRUE)
  male.btdist <- coldist(male.bt, subset = 'back', vis = 'tetra', achro = TRUE)

## BEE visual system ##

# This requires we 'manually' model bee receptor sensitivities as pavo doesn't 
# have a bee built in. To do this properly you need to know something about how
# the animal's visual system is set up, (receptor densities, weber fractions ('noise'),
# filtering pigments/oil droplets etc.)

# Sensmodel works from Govardovskii et al's (2000) template. Honeybee receptor
# sensitivities are from the lit. Integrate = TRUE (default) if you're doing 
# visual modelling with the sensitivities. 
  beesense <- sensmodel(c(344, 436, 556), integrate = TRUE)  
  names(beesense) <- c('wl', 's', 'm', 'l')
  plot(beesense, ylab = 'Absorbance')  # Have a look at it

# Bee visual system modelling
  female.bee <- vismodel(female.RN, visual = beesense, achromatic = 'none', bkg = back, illum = 'D65', relative = FALSE, qcatch = 'fi')
  male.bee <- vismodel(male.RN, visual = beesense, achromatic = 'none', illum = 'D65', relative = FALSE, qcatch = 'fi')

# Calculate chromatic and achromatic contrasts (colour-distances)
# the 'n's' are the relative densities of photoreceptor types and signal-to-noise
# ratio for honeybees - both from the literature
  female.beedist <- coldist(female.bee, subset = 'back', vis = 'tri', 
                            n1 = 1, n2 = 0.471, n3 = 4.412, achro = TRUE, v = 0.13)
  male.beedist <- coldist(male.bee, subset = 'back', vis = 'tri', 
                          n1 = 1, n2 = 0.471, n3 = 4.412, achro = TRUE, v = 0.13)

## MANTID visual system ##

# To the best of our (limited) knowledge mantids are monochromats and are 
# truly colour-blind (not necessarily the same thing). Also, we're using the weber 
# fraction value from honey bees of v = 0.13 as we don't know enough about noise
# in mantid photoreceptors. This obviously limits our inferences regarding discriminability.

# Pavo doesn't currently handle monochromats so we have to 'trick' it into 
# calculating achromatic contrast by inputting mantid sensitivities as if they were dichromats. 
# This is fine - we just ignore chromatic contrasts as they'll be meaningless/non-existant. 
# Pavo calculates achromatic contrast from the last column in the vismodel data frame, 
# which in this case will simply be the sensitivity function of the one mantid photoreceptor. 
# It's ugly - but it works!

# Model mantid receptor sensitivity
  mantidsense <- sensmodel(515, integrate = TRUE)
  names(mantidsense) <- c('wl', 'm')
  plot(mantidsense)

# Create a hypothetical dichromat mantid
  mantidsense2 <- cbind(mantidsense, mantidsense$m)
  names(mantidsense2) <- c('wl', 's', 'm')

# Mantid visual modelling
  female.mantid <- vismodel(female.RN, visual = mantidsense2, achromatic = 'none', 
                            illum = 'D65', relative = FALSE, qcatch = 'fi')
  male.mantid <- vismodel(male.RN, visual = mantidsense2, achromatic = 'none', 
                          illum = 'D65', relative = FALSE, qcatch = 'fi')

# Calculate chromatic and achromatic contrasts (colour-distances).
  female.mantiddist <- coldist(female.mantid, subset = 'back', vis = 'di', 
                               n1 = 1, n2 = 1, n4 = 1, achro = TRUE, v = 0.13)
  male.mantiddist <- coldist(male.mantid, subset = 'back', vis = 'di', 
                             n1 = 1, n2 = 1, n4 = 1, achro = TRUE, v = 0.13)

#---------------------------------------#
       # Plotting contrasts

# This is just an ugly way of getting all the summary contrast data (JND's: mean +- s.d) 
# into 'long' format for ggplot2. I'm sure there's a better way!
# You could also just not use ggplot2, which is often wise, but I find it easier
# to play with error bars in ggplot2

# Males
  male.ch.mean <- rbind(mean(male.btdist$dS), mean(male.beedist$dS), mean(male.mantiddist$dS))
  male.ch.sd <- rbind(sd(male.btdist$dS), sd(male.beedist$dS), sd(male.mantiddist$dS))
  male.plot1  <- as.data.frame(cbind(male.ch.mean, male.ch.sd))
  male.plot1$contrast <- 'chromatic'
  male.plot1$system <- c('blue tit', 'bee', 'mantid')
  
  male.ach.mean <- rbind(mean(male.btdist$dL), mean(male.beedist$dL), mean(male.mantiddist$dL))
  male.ach.sd <- rbind(sd(male.btdist$dL), sd(male.beedist$dL), sd(male.mantiddist$dL))
  male.plot2  <- as.data.frame(cbind(male.ach.mean, male.ach.sd))
  male.plot2$contrast <- 'achromatic'
  male.plot2$system <- c('blue tit', 'bee', 'mantid')
  
  male.plot <- rbind(male.plot1, male.plot2)
  names(male.plot) <- c('mean', 'sd', 'contrast', 'system')

# Females
  female.ch.mean <- rbind(mean(female.btdist$dS), mean(female.beedist$dS), mean(female.mantiddist$dS))
  female.ch.sd <- rbind(sd(female.btdist$dS), sd(female.beedist$dS), sd(female.mantiddist$dS))
  female.plot1  <- as.data.frame(cbind(female.ch.mean, female.ch.sd))
  female.plot1$contrast <- 'chromatic'
  female.plot1$system <- c('blue tit', 'bee', 'mantid')
  
  female.ach.mean <- rbind(mean(female.btdist$dL), mean(female.beedist$dL), mean(female.mantiddist$dL))
  female.ach.sd <- rbind(sd(female.btdist$dL), sd(female.beedist$dL), sd(female.mantiddist$dL))
  female.plot2  <- as.data.frame(cbind(female.ach.mean, female.ach.sd))
  female.plot2$contrast <- 'achromatic'
  female.plot2$system <- c('blue tit', 'bee', 'mantid')
  
  female.plot <- rbind(female.plot1, female.plot2)
  names(female.plot) <- c('mean', 'sd', 'contrast', 'system')

# Reorder body parts for plotting. Again, this is ugly.
  female.plot$contrast <- factor(female.plot$contrast, levels = c('chromatic', 'achromatic')) 
  male.plot$contrast <- factor(male.plot$contrast, levels = c('chromatic', 'achromatic')) 

# Plot 'em

# Females
  ggfem <- ggplot(female.plot, aes(x = system, y = mean, fill = contrast)) + 
    geom_bar(position = position_dodge(), width = 0.85, stat="identity",
             colour = "black", # Use black outlines,
             size = 0.3) +      # Thinner lines
    geom_errorbar(aes(ymin = mean - sd, ymax = mean + sd),
                  size =.3,    # Thinner lines
                  width =.2,
                  position = position_dodge(.85)) +
    xlab('Visual system') +
    ylab('Contrast (JND)') +
    scale_y_continuous(expand = c(0, 0), limits = c(0, 50)) +
    geom_hline(aes(yintercept = 1), linetype = 'dashed') +  # typical 'threshold' of 1 JND
    theme_bw() +
    theme(axis.line = element_line(colour = 'black'),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = element_blank(),
          panel.background = element_blank(),
          axis.title.x = element_text(vjust = 0))

# Males
  ggmale <- ggplot(male.plot, aes(x = system, y = mean, fill = contrast)) + 
    geom_bar(position = position_dodge(), width = 0.85, stat="identity",
             colour = "black", # Use black outlines,
             size = 0.3) +      # Thinner lines
    geom_errorbar(aes(ymin = mean - sd, ymax = mean + sd),
                  size =.3,    # Thinner lines
                  width =.2,
                  position = position_dodge(.85)) +
    xlab('Visual system') +
    ylab('Contrast (JND)') +
    scale_y_continuous(expand = c(0, 0), limits = c(0, 50)) +
    geom_hline(aes(yintercept = 1), linetype = 'dashed') +
    theme_bw() +
    theme(axis.line = element_line(colour = 'black'),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = element_blank(),
          panel.background = element_blank(),
          axis.title.x = element_text(vjust = 0))

# Take a look at contrast levels. Are our butterflies discriminable against a green
# vegetation background in the visual system of bees, blue tits, and mantids?
# Yes, of course.
  ggfem
  ggmale

##--------------------------------------------------------------------------##
                ## Visual modelling - TCS

# The tetrahedral colourspace is often used to explore 'perceptual' questions
# such as differences between whole-colour patches, for example. It can't really
# explore questions of 'discrimination' on it's own in the way that the receptor-noise
# model can (although see Stoddard & Stevens 2010, Avian Vision...for a nice use), 
# but it's more appropriate for addressing questions when stimuli are obviously 
# chromatically different (i.e. at 'supra-threshold' distances). As the name suggests, 
# it's tetrahedral (though there is a trichromat version - not currently implemented),
# so it's mostly useful for avian visual modelling. I suspect it is possible to 
# just set one receptor to '0' and collaspe it down for a trichomat in pavo.
# Methods based on Endler & Mielke 2005, Stoddard & Prum 2008, depending on the options used.

# 'vismodel' again, only relative must be TRUE. Again, we're using the blue tit visual system. 
# I know I'm re-using variable names...
  female.bt <- vismodel(butt.female, visual = 'bt', achromatic = 'bt.dc', 
                        illum = 'D65', relative = TRUE, qcatch = 'fi')
  male.bt <- vismodel(butt.male, visual = 'bt', achromatic = 'bt.dc', 
                      illum = 'D65', relative = TRUE, qcatch = 'fi')
  plant.bt <- vismodel(plant, visual = 'bt', achromatic = 'bt.dc', 
                       illum = 'D65', relative = TRUE, qcatch = 'fi')

# So instead of 'coldist' we're using 'tcs'. This calculates the location of every 
# reflectance spectrum in tetrahedral colour-space (the blue tit space here).
# It returns a bunch of different variables that describe the location of each point, 
# which are your measures of 'hue' and 'saturation'.
  female.bt.tcs <- tcs(female.bt)
  male.bt.tcs <- tcs(male.bt)
  plant.bt.tcs <- tcs(plant.bt)

# Have a look at what it calculates. 'h.theta' & 'h.phi' are you two hue angles,
# 'r.vec' is a measure of saturation (distance from achromatic center of the 
# tetrahedron) etc.
  head(female.bt.tcs)

# Some summary variables of each whole group
  summary(female.bt.tcs)
  summary(male.bt.tcs)
  summary(plant.bt.tcs)

# Visualise point in the 3d tetrahedral space. There's currently no easy way
# to save these 3d plots, but I think it's being overhauled in the next version.
  tcsplot(female.bt.tcs, col = 'blue', size = 0.01) # Females
  tcspoints(male.bt.tcs, col = 'red', size = 0.01)  # Add male points
  tcspoints(plant.bt.tcs, col = 'forestgreen', size = 0.01)  # Add male points

# Visualise point in the 3d tetrahedral space w/ volume hulls
  tcsvol(female.bt.tcs, col = 'blue') # Females
  tcsvol(male.bt.tcs, col = 'red')  # Add male points
  tcsvol(plant.bt.tcs, col = 'forestgreen')  # Add male points

# A 2d 'projection plot' of the same thing
  projplot(female.bt.tcs, col = 'blue', pch = 20, cex = 1.8) # Females
  points(male.bt.tcs, col = 'red', pch = 20, cex = 1.8)  # Add male points
  points(plant.bt.tcs, col = 'forestgreen', pch = 20, cex = 1.8)  # Add male points

# Calculate the the 'volume occupied' and 'volume overlap' between sets of points 
# in the tetrahedral space. Can be useful for all sorts of questions. Here we're using 
# the example of male & female butterflies.
  voloverlap(female.bt.tcs, male.bt.tcs, plot = TRUE)  
# Interesting, males & females don't overlap at all - I wouldn't have expected that. 
